@@ -24,15 +24,11 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
-
-#include "c_lib.h"
-#include "c_private.h"
-
 #include "csync_exclude.h"
 
-#include "../version.h"
 #include "common/filesystembase.h"
 #include "common/utility.h"
+#include "common/version.h"
 
 #include <QString>
 #include <QFileInfo>
@@ -200,23 +196,16 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QStringRef &path, bool ex
 
     // Filter out characters not allowed in a filename on windows
     // see https://docs.microsoft.com/en-us/windows/desktop/fileio/naming-a-file
-    for (auto p : path) {
-        ushort c = p.unicode();
+    for (auto c : path) {
         if (c < 32) {
             return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
         }
-        switch (c) {
-        case '\\':
-        case ':':
-        case '?':
-        case '*':
-        case '"':
-        case '>':
-        case '<':
-        case '|':
+        if (std::find_if(
+                OCC::FileSystem::IllegalFilenameCharsWindows.begin(), OCC::FileSystem::IllegalFilenameCharsWindows.end(), [c](const auto illegal) {
+                    return c == illegal;
+                })
+            != OCC::FileSystem::IllegalFilenameCharsWindows.end()) {
             return CSYNC_FILE_EXCLUDE_INVALID_CHAR;
-        default:
-            break;
         }
     }
 #endif
@@ -238,7 +227,7 @@ static CSYNC_EXCLUDE_TYPE _csync_excluded_common(const QStringRef &path, bool ex
 using namespace OCC;
 
 ExcludedFiles::ExcludedFiles()
-    : _clientVersion(MIRALL_VERSION_MAJOR, MIRALL_VERSION_MINOR, MIRALL_VERSION_PATCH)
+    : _clientVersion(OCC::Version::version())
 {
     // Windows used to use PathMatchSpec which allows *foo to match abc/deffoo.
     _wildcardsMatchSlash = Utility::isWindows();
@@ -277,7 +266,7 @@ void ExcludedFiles::setWildcardsMatchSlash(bool onoff)
     prepare();
 }
 
-void ExcludedFiles::setClientVersion(ExcludedFiles::Version version)
+void ExcludedFiles::setClientVersion(const QVersionNumber &version)
 {
     _clientVersion = version;
 }
@@ -321,7 +310,7 @@ bool ExcludedFiles::versionDirectiveKeepNextLine(const QByteArray &directive) co
     if (argVersions.size() != 3)
         return true;
 
-    auto argVersion = std::make_tuple(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
+    const auto argVersion = QVersionNumber(argVersions[0].toInt(), argVersions[1].toInt(), argVersions[2].toInt());
     if (op == "<=")
         return _clientVersion <= argVersion;
     if (op == "<")
