@@ -15,35 +15,36 @@
  */
 
 #include "application.h"
+#include "application_p.h"
 
 #include <iostream>
 #include <random>
 
-#include "config.h"
-#include "common/asserts.h"
 #include "account.h"
+#include "accountmanager.h"
 #include "accountstate.h"
+#include "clientproxy.h"
+#include "common/asserts.h"
+#include "common/version.h"
+#include "common/vfs.h"
+#include "configfile.h"
 #include "connectionvalidator.h"
+#include "creds/abstractcredentials.h"
+#include "csync_exclude.h"
 #include "folder.h"
 #include "folderman.h"
-#include "logger.h"
 #include "logbrowser.h"
-#include "configfile.h"
-#include "socketapi/socketapi.h"
-#include "sslerrordialog.h"
-#include "theme.h"
-#include "clientproxy.h"
-#include "sharedialog.h"
-#include "accountmanager.h"
-#include "creds/abstractcredentials.h"
-#include "updater/ocupdater.h"
-#include "owncloudsetupwizard.h"
-#include "version.h"
-#include "csync_exclude.h"
-#include "common/vfs.h"
+#include "logger.h"
 #include "settingsdialog.h"
+#include "sharedialog.h"
+#include "socketapi/socketapi.h"
+#include "theme.h"
 #include "translations.h"
 
+#ifdef WITH_AUTO_UPDATER
+#include "updater/ocupdater.h"
+#endif
+
 #include "config.h"
 
 #if defined(Q_OS_WIN)
@@ -54,9 +55,14 @@
 #include <libcrashreporter-handler/Handler.h>
 #endif
 
-#include <QTranslator>
+#include <QCommandLineParser>
+#include <QDir>
+#include <QLibraryInfo>
 #include <QMenu>
 #include <QMessageBox>
+#include <QScopeGuard>
+#include <QTranslator>
+
 #pragma push_macro("QT_DISABLE_DEPRECATED_BEFORE")
 #undef QT_DISABLE_DEPRECATED_BEFORE
 #define QT_DISABLE_DEPRECATED_BEFORE 0
@@ -118,7 +124,7 @@ void migrateConfigFile(const QCoreApplication *app)
                     const auto filesList = QDir(oldDir).entryInfoList(QDir::Files);
                     qCInfo(lcApplication) << Q_FUNC_INFO << "Will move the individual files" << filesList;
                     for (const auto &fileInfo : filesList) {
-                        if (!QFile::rename(fileInfo.canonicalFilePath(), confDir + "/" + fileInfo.fileName())) {
+                        if (!QFile::rename(fileInfo.canonicalFilePath(), confDir + QLatin1Char('/') + fileInfo.fileName())) {
                             qCWarning(lcApplication) << Q_FUNC_INFO << "Fallback move of " << fileInfo.fileName() << "also failed";
                         } else {
                             // we found a suitable config directory to migrate, hence we can stop here
@@ -157,7 +163,7 @@ bool Application::configVersionMigration()
 
     // Did the client version change?
     // (The client version is adjusted further down)
-    bool versionChanged = configFile.clientVersionString() != MIRALL_VERSION_STRING;
+    const bool versionChanged = QVersionNumber::fromString(configFile.clientVersionString()) != OCC::Version::version();
 
     // We want to message the user either for destructive changes,
     // or if we're ignoring something and the client version changed.
@@ -178,7 +184,7 @@ bool Application::configVersionMigration()
 
         QMessageBox box(
             QMessageBox::Warning,
-            APPLICATION_SHORTNAME,
+            Theme::instance()->appNameGUI(),
             tr("Some settings were configured in newer versions of this client and "
                "use features that are not available in this version.<br>"
                "<br>"
@@ -195,7 +201,7 @@ bool Application::configVersionMigration()
             return false;
         }
 
-        auto settings = ConfigFile::settingsWithGroup("foo");
+        auto settings = ConfigFile::settingsWithGroup(QStringLiteral("foo"));
         settings->endGroup();
 
         // Wipe confusing keys from the future, ignore the others
@@ -203,7 +209,7 @@ bool Application::configVersionMigration()
             settings->remove(badKey);
     }
 
-    configFile.setClientVersionString(MIRALL_VERSION_STRING);
+    configFile.setClientVersionString(OCC::Version::version().toString());
     return true;
 }
 
@@ -217,33 +223,28 @@ ownCloudGui *Application::gui() const
     return _gui;
 }
 
-Application::Application(int &argc, char **argv)
+Application::Application(int &argc, char **argv, Platform *platform)
     : SharedTools::QtSingleApplication(Theme::instance()->appName(), argc, argv)
     , _gui(nullptr)
     , _theme(Theme::instance())
-    , _logExpire(0)
     , _logFlush(false)
     , _logDebug(false)
     , _userTriggeredConnect(false)
     , _debugMode(false)
 {
-    qsrand(std::random_device()());
-
-#ifdef Q_OS_WIN
-    // Ensure OpenSSL config file is only loaded from app directory
-    QString opensslConf = QCoreApplication::applicationDirPath() + QString("/openssl.cnf");
-    qputenv("OPENSSL_CONF", opensslConf.toLocal8Bit());
-#endif
+    platform->setApplication(this);
 
     // TODO: Can't set this without breaking current config paths
     //    setOrganizationName(QLatin1String(APPLICATION_VENDOR));
-    setOrganizationDomain(QLatin1String(APPLICATION_REV_DOMAIN));
+    setOrganizationDomain(QStringLiteral(APPLICATION_REV_DOMAIN));
     setApplicationName(_theme->appName());
     setWindowIcon(_theme->applicationIcon());
 
     // migrate old configuration files if necessary
     migrateConfigFile(this);
 
+    platform->migrate();
+
     // needed during commandline options parsing
     setApplicationVersion(_theme->versionSwitchOutput());
 
@@ -267,20 +268,30 @@ Application::Application(int &argc, char **argv)
     setupLogging();
     setupTranslations();
 
+    qCInfo(lcApplication) << "Plugin search paths:" << libraryPaths();
+
+    // Check vfs plugins
+    if (Theme::instance()->showVirtualFilesOption() && bestAvailableVfsMode() == Vfs::Off) {
+        qCWarning(lcApplication) << "Theme wants to show vfs mode, but no vfs plugins are available";
+    }
+    if (isVfsPluginAvailable(Vfs::WindowsCfApi))
+        qCInfo(lcApplication) << "VFS windows plugin is available";
+    if (isVfsPluginAvailable(Vfs::WithSuffix))
+        qCInfo(lcApplication) << "VFS suffix plugin is available";
+
     if (!configVersionMigration()) {
         return;
     }
 
     ConfigFile cfg;
-    // The timeout is initialized with an environment variable, if not, override with the value from the config
-    if (!AbstractNetworkJob::httpTimeout)
-        AbstractNetworkJob::httpTimeout = cfg.timeout();
 
-#if defined(OC_PLUGIN_DIR) && defined(Q_OS_LINUX)
-    const QString extraPluginDir = QDir(QApplication::applicationDirPath()).filePath(QStringLiteral(OC_PLUGIN_DIR));
-    qCInfo(lcApplication) << "Adding extra plugin search path:" << extraPluginDir;
-    this->addLibraryPath(extraPluginDir);
-#endif
+    // this should be called once during application startup to make sure we don't miss any messages
+    cfg.configureHttpLogging();
+
+    // The timeout is initialized with an environment variable, if not, override with the value from the config
+    if (AbstractNetworkJob::httpTimeout == AbstractNetworkJob::DefaultHttpTimeout) {
+        AbstractNetworkJob::httpTimeout = cfg.timeout();
+    }
 
     // Check vfs plugins
     if (Theme::instance()->showVirtualFilesOption() && bestAvailableVfsMode() == Vfs::Off) {
@@ -337,14 +348,14 @@ Application::Application(int &argc, char **argv)
     _proxy.setupQtProxyFromConfig(); // folders have to be defined first, than we set up the Qt proxy.
 
     // Enable word wrapping of QInputDialog (#4197)
-    setStyleSheet("QInputDialog QLabel { qproperty-wordWrap:1; }");
+    setStyleSheet(QStringLiteral("QInputDialog QLabel { qproperty-wordWrap:1; }"));
 
     connect(AccountManager::instance(), &AccountManager::accountAdded,
         this, &Application::slotAccountStateAdded);
     connect(AccountManager::instance(), &AccountManager::accountRemoved,
         this, &Application::slotAccountStateRemoved);
     for (const auto &ai : AccountManager::instance()->accounts()) {
-        slotAccountStateAdded(ai.data());
+        slotAccountStateAdded(ai);
     }
 
     connect(FolderMan::instance()->socketApi(), &SocketApi::shareCommandReceived,
@@ -352,7 +363,7 @@ Application::Application(int &argc, char **argv)
 
     // startup procedure.
     connect(&_checkConnectionTimer, &QTimer::timeout, this, &Application::slotCheckConnection);
-    _checkConnectionTimer.setInterval(ConnectionValidator::DefaultCallingIntervalMsec); // check for connection every 32 seconds.
+    _checkConnectionTimer.setInterval(ConnectionValidator::DefaultCallingInterval);
     _checkConnectionTimer.start();
     // Also check immediately
     QTimer::singleShot(0, this, &Application::slotCheckConnection);
@@ -361,6 +372,7 @@ Application::Application(int &argc, char **argv)
     connect(&_networkConfigurationManager, &QNetworkConfigurationManager::configurationChanged,
         this, &Application::slotSystemOnlineConfigurationChanged);
 
+#ifdef WITH_AUTO_UPDATER
     // Update checks
     UpdaterScheduler *updaterScheduler = new UpdaterScheduler(this);
     connect(updaterScheduler, &UpdaterScheduler::updaterAnnouncement,
@@ -369,6 +381,7 @@ Application::Application(int &argc, char **argv)
         });
     connect(updaterScheduler, &UpdaterScheduler::requestRestart,
         _folderManager.data(), &FolderMan::slotScheduleAppRestart);
+#endif
 
     // Cleanup at Quit.
     connect(this, &QCoreApplication::aboutToQuit, this, &Application::slotCleanup);
@@ -386,21 +399,8 @@ Application::~Application()
     AccountManager::instance()->shutdown();
 }
 
-void Application::slotAccountStateRemoved(AccountState *accountState)
+void Application::slotAccountStateRemoved() const
 {
-    if (_gui) {
-        disconnect(accountState, &AccountState::stateChanged,
-            _gui.data(), &ownCloudGui::slotAccountStateChanged);
-        disconnect(accountState->account().data(), &Account::serverVersionChanged,
-            _gui.data(), &ownCloudGui::slotTrayMessageIfServerUnsupported);
-    }
-    if (_folderManager) {
-        disconnect(accountState, &AccountState::stateChanged,
-            _folderManager.data(), &FolderMan::slotAccountStateChanged);
-        disconnect(accountState->account().data(), &Account::serverVersionChanged,
-            _folderManager.data(), &FolderMan::slotServerVersionChanged);
-    }
-
     // if there is no more account, show the wizard.
     if (_gui && AccountManager::instance()->accounts().isEmpty()) {
         // allow to add a new account if there is non any more. Always think
@@ -409,16 +409,23 @@ void Application::slotAccountStateRemoved(AccountState *accountState)
     }
 }
 
-void Application::slotAccountStateAdded(AccountState *accountState)
+void Application::slotAccountStateAdded(AccountStatePtr accountState) const
 {
-    connect(accountState, &AccountState::stateChanged,
+    // Hook up the GUI slots to the account state's signals:
+    connect(accountState.data(), &AccountState::stateChanged,
         _gui.data(), &ownCloudGui::slotAccountStateChanged);
     connect(accountState->account().data(), &Account::serverVersionChanged,
-        _gui.data(), &ownCloudGui::slotTrayMessageIfServerUnsupported);
-    connect(accountState, &AccountState::stateChanged,
+        _gui.data(), [account = accountState->account().data(), this] {
+            _gui->slotTrayMessageIfServerUnsupported(account);
+        });
+
+    // Hook up the folder manager slots to the account state's signals:
+    connect(accountState.data(), &AccountState::stateChanged,
         _folderManager.data(), &FolderMan::slotAccountStateChanged);
     connect(accountState->account().data(), &Account::serverVersionChanged,
-        _folderManager.data(), &FolderMan::slotServerVersionChanged);
+        _folderManager.data(), [account = accountState->account().data()] {
+            FolderMan::instance()->slotServerVersionChanged(account);
+        });
 
     _gui->slotTrayMessageIfServerUnsupported(accountState->account().data());
 }
@@ -435,7 +442,7 @@ void Application::slotCleanup()
 // FIXME: This is not ideal yet since a ConnectionValidator might already be running and is in
 // progress of timing out in some seconds.
 // Maybe we need 2 validators, one triggered by timer, one by network configuration changes?
-void Application::slotSystemOnlineConfigurationChanged(QNetworkConfiguration cnf)
+void Application::slotSystemOnlineConfigurationChanged(const QNetworkConfiguration &cnf)
 {
     if (cnf.state() & QNetworkConfiguration::Active) {
         QMetaObject::invokeMethod(this, "slotCheckConnection", Qt::QueuedConnection);
@@ -487,44 +494,50 @@ void Application::slotShowGuiMessage(const QString &title, const QString &messag
 }
 
 
-void Application::slotownCloudWizardDone(int res)
+AccountStatePtr Application::addNewAccount(AccountPtr newAccount)
 {
-    AccountManager *accountMan = AccountManager::instance();
-    FolderMan *folderMan = FolderMan::instance();
+    auto *accountMan = AccountManager::instance();
 
-    // During the wizard, scheduling of new syncs is disabled
-    folderMan->setSyncEnabled(true);
+    // first things first: we need to add the new account
+    auto accountStatePtr = accountMan->addAccount(newAccount);
 
-    if (res == QDialog::Accepted) {
-        // Check connectivity of the newly created account
-        _checkConnectionTimer.start();
-        slotCheckConnection();
+    // check connectivity of the newly created account
+    _checkConnectionTimer.start();
+    slotCheckConnection();
 
-        // If one account is configured: enable autostart
-        bool shouldSetAutoStart = (accountMan->accounts().size() == 1);
+    // if one account is configured: enable autostart
+    bool shouldSetAutoStart = (accountMan->accounts().size() == 1);
 #ifdef Q_OS_MAC
-        // Don't auto start when not being 'installed'
-        shouldSetAutoStart = shouldSetAutoStart
-            && QCoreApplication::applicationDirPath().startsWith("/Applications/");
+    // Don't auto start when not being 'installed'
+    shouldSetAutoStart = shouldSetAutoStart
+        && QCoreApplication::applicationDirPath().startsWith(QLatin1String("/Applications/"));
 #endif
-        if (shouldSetAutoStart) {
-            Utility::setLaunchOnStartup(_theme->appName(), _theme->appNameGUI(), true);
-        }
-
-        _gui->slotShowSettings();
+    if (shouldSetAutoStart) {
+        Utility::setLaunchOnStartup(_theme->appName(), _theme->appNameGUI(), true);
     }
+
+    // showing the UI to show the user that the account has been added successfully
+    _gui->slotShowSettings();
+
+    return accountStatePtr;
 }
 
 void Application::setupLogging()
 {
     // might be called from second instance
     auto logger = Logger::instance();
-    logger->setLogFile(_logFile);
-    logger->setLogDir(_logDir);
-    logger->setLogExpire(_logExpire);
+    // call setLogFlush first, other log settings might already imply flushing
+    // so setting it false in the end will have undesired results.
     logger->setLogFlush(_logFlush);
+
+    if (!_logDir.isEmpty()) {
+        logger->setLogDir(_logDir);
+    }
+    if (!_logFile.isEmpty()) {
+        Q_ASSERT(_logDir.isEmpty());
+        logger->setLogFile(_logFile);
+    }
     logger->setLogDebug(_logDebug);
-    logger->enterNextLogFile();
 
     // Possibly configure logging from config file
     LogBrowser::setupLoggingFromConfig();
@@ -542,12 +555,12 @@ void Application::slotUseMonoIconsChanged(bool)
 
 void Application::slotParseMessage(const QString &msg, QObject *)
 {
-    if (msg.startsWith(QLatin1String("MSG_PARSEOPTIONS:"))) {
-        const int lengthOfMsgPrefix = 17;
-        QStringList options = msg.mid(lengthOfMsgPrefix).split(QLatin1Char('|'));
+    qCInfo(lcApplication) << Q_FUNC_INFO << msg;
+    if (msg.startsWith(ApplicationPrivate::msgParseOptionsC())) {
+        QStringList options = msg.mid(ApplicationPrivate::msgParseOptionsC().size()).split(QLatin1Char('|'));
+        // TODO: don't modify the application state in the parser
         _showSettings = false;
         parseOptions(options);
-        setupLogging();
         if (_showSettings) {
             _gui->slotShowSettings();
         }
@@ -564,7 +577,7 @@ static void displayHelpText(const QString &t, std::ostream &stream = std::cout)
     stream << qUtf8Printable(t) << std::endl;
 #ifdef Q_OS_WIN
     // No console on Windows.
-    QString spaces(80, ' '); // Add a line of non-wrapped space to make the messagebox wide enough.
+    QString spaces(80, QLatin1Char(' ')); // Add a line of non-wrapped space to make the messagebox wide enough.
     QString text = QStringLiteral("<qt><pre style='white-space:pre-wrap'>")
         + t.toHtmlEscaped() + QStringLiteral("</pre><pre>") + spaces + QStringLiteral("</pre></qt>");
     QMessageBox::information(0, Theme::instance()->appNameGUI(), text);
@@ -578,12 +591,12 @@ void Application::parseOptions(const QStringList &arguments)
     QString descriptionText;
     QTextStream descriptionTextStream(&descriptionText);
 
-    descriptionTextStream << tr("%1 version %2\r\nFile synchronization desktop utility.").arg(_theme->appName(), _theme->version()) << endl;
+    descriptionTextStream << tr("%1 version %2\r\nFile synchronization desktop utility.").arg(_theme->appName(), OCC::Version::displayString()) << endl;
 
     if (_theme->appName() == QLatin1String("ownCloud")) {
         descriptionTextStream << endl
                               << endl
-                              << tr("For more information, see http://www.owncloud.org");
+                              << tr("For more information, see %1", "link to homepage").arg(QStringLiteral("https://www.owncloud.com"));
     }
 
     parser.setApplicationDescription(descriptionText);
@@ -597,20 +610,19 @@ void Application::parseOptions(const QStringList &arguments)
         return option;
     };
 
-    auto showSettingsOption = addOption({ { "s", "showsettings" }, tr("Show the settings dialog while starting.") });
-    auto quitInstanceOption = addOption({ { "q", "quit" }, tr("Quit the running instance.") });
-    auto logFileOption = addOption({ "logfile", tr("Write log to file (use - to write to stdout)."), "filename" });
-    auto logDirOption = addOption({ "logdir", tr("Write each sync log output in a new file in folder."), "name" });
-    auto logExpireOption = addOption({ "logexpire", tr("Remove logs older than <hours> hours (to be used with --logdir)."), "hours" });
-    auto logFlushOption = addOption({ "logflush", tr("Flush the log file after every write.") });
-    auto logDebugOption = addOption({ "logdebug", tr("Output debug-level messages in the log.") });
-    auto languageOption = addOption({ "language", tr("Override UI language."), "language" });
-    auto listLanguagesOption = addOption({ "list-languages", tr("Override UI language.") });
-    auto confDirOption = addOption({ "confdir", tr("Use the given configuration folder."), "dirname" });
-    auto debugOption = addOption({ "debug", tr("Enable debug mode.") });
+    auto showSettingsOption = addOption({ { QStringLiteral("s"), QStringLiteral("showsettings") }, tr("Show the settings dialog while starting.") });
+    auto quitInstanceOption = addOption({ { QStringLiteral("q"), QStringLiteral("quit") }, tr("Quit the running instance.") });
+    auto logFileOption = addOption({ QStringLiteral("logfile"), tr("Write log to file (use - to write to stdout)."), QStringLiteral("filename") });
+    auto logDirOption = addOption({ QStringLiteral("logdir"), tr("Write each sync log output in a new file in folder."), QStringLiteral("name") });
+    auto logFlushOption = addOption({ QStringLiteral("logflush"), tr("Flush the log file after every write.") });
+    auto logDebugOption = addOption({ QStringLiteral("logdebug"), tr("Output debug-level messages in the log.") });
+    auto languageOption = addOption({ QStringLiteral("language"), tr("Override UI language."), QStringLiteral("language") });
+    auto listLanguagesOption = addOption({ QStringLiteral("list-languages"), tr("Override UI language.") });
+    auto confDirOption = addOption({ QStringLiteral("confdir"), tr("Use the given configuration folder."), QStringLiteral("dirname") });
+    auto debugOption = addOption({ QStringLiteral("debug"), tr("Enable debug mode.") });
 
     // virtual file system parameters (optional)
-    parser.addPositionalArgument("vfs file", tr("Virtual file system file to be opened (optional)."), { tr("[<vfs file>]") });
+    parser.addPositionalArgument(QStringLiteral("vfs file"), tr("Virtual file system file to be opened (optional)."), { tr("[<vfs file>]") });
 
     parser.process(arguments);
 
@@ -625,11 +637,12 @@ void Application::parseOptions(const QStringList &arguments)
         _logFile = parser.value(logFileOption);
     }
     if (parser.isSet(logDirOption)) {
+        if (parser.isSet(logFileOption)) {
+            displayHelpText(tr("--logfile and --logdir are mutually exclusive"));
+            std::exit(1);
+        }
         _logDir = parser.value(logDirOption);
     }
-    if (parser.isSet(logExpireOption)) {
-        _logExpire = std::chrono::hours(parser.value(logExpireOption).toInt());
-    }
     if (parser.isSet(logFlushOption)) {
         _logFlush = true;
     }
@@ -661,7 +674,7 @@ void Application::parseOptions(const QStringList &arguments)
     if (parser.isSet(listLanguagesOption)) {
         auto availableTranslations = Translations::listAvailableTranslations().toList();
         availableTranslations.sort(Qt::CaseInsensitive);
-        displayHelpText(tr("Available translations: %1").arg(availableTranslations.join(", ")));
+        displayHelpText(tr("Available translations: %1").arg(availableTranslations.join(QStringLiteral(", "))));
         std::exit(1);
     }
 
@@ -686,10 +699,10 @@ QString substLang(const QString &lang)
 
     // Simplified Chinese
     if (lang == QLatin1String("zh_Hans"))
-        return QLatin1String("zh_CN");
+        return QStringLiteral("zh_CN");
     // Traditional Chinese
     if (lang == QLatin1String("zh_Hant"))
-        return QLatin1String("zh_TW");
+        return QStringLiteral("zh_TW");
     return lang;
 }
 
